#include <fcitx-utils/i18n.h>

namespace {
[[maybe_unused]] constexpr std::string_view OpenCCProfileName[] = {
    NC_("OpenCC Profile", "Traditional Chinese (Taiwan standard) to Simplified Chinese"),
    NC_("OpenCC Profile", "Traditional Chinese (Taiwan standard) to Traditional Chinese"),
    NC_("OpenCC Profile", "Traditional Chinese (Taiwan standard) to Simplified Chinese (with phrases)"),
    NC_("OpenCC Profile", "Traditional Chinese to Traditional Chinese (Taiwan standard)"),
    NC_("OpenCC Profile", "Simplified Chinese to Traditional Chinese (Taiwan standard, with phrases)"),
    NC_("OpenCC Profile", "Simplified Chinese to Traditional Chinese (Taiwan standard)"),
    NC_("OpenCC Profile", "Simplified Chinese to Traditional Chinese"),
    NC_("OpenCC Profile", "Traditional Chinese (Hong Kong variant) to Simplified Chinese"),
    NC_("OpenCC Profile", "New Japanese Kanji (Shinjitai) to Traditional Chinese Characters (Kyūjitai)"),
    NC_("OpenCC Profile", "Traditional Chinese to Simplified Chinese"),
    NC_("OpenCC Profile", "Traditional Chinese Characters (Kyūjitai) to New Japanese Kanji (Shinjitai)"),
    NC_("OpenCC Profile", "Traditional Chinese (Hong Kong variant) to Traditional Chinese"),
    NC_("OpenCC Profile", "Traditional Chinese to Traditional Chinese (Hong Kong variant)"),
    NC_("OpenCC Profile", "Simplified Chinese to Traditional Chinese (Hong Kong variant)"),
};
}
